/*
 * Copyright 2017-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.onosproject.fpcagent.util.eventstream;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.http.HttpResponse;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.apache.http.impl.nio.client.HttpAsyncClients;
import org.apache.http.nio.IOControl;
import org.apache.http.nio.client.methods.AsyncCharConsumer;
import org.apache.http.nio.client.methods.HttpAsyncMethods;
import org.apache.http.nio.protocol.HttpAsyncRequestProducer;
import org.apache.http.protocol.HttpContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.CharBuffer;
import java.util.AbstractMap;

/**
 * A HTTP ASYNC_CLIENT that sends a request to a FPC Client to initiate the request stream.
 */
public class RequestServer {
    private static final Logger log = LoggerFactory.getLogger(RequestServer.class);

    private static final CloseableHttpAsyncClient ASYNC_CLIENT = HttpAsyncClients.createDefault();
    protected String clientUri;

    /**
     * Send HttpRequest to Client.
     *
     * @param uri - FPC Client Uri
     */
    public void connectToClient(String uri) {
        this.clientUri = uri;
        try {
            ASYNC_CLIENT.start();
            HttpAsyncRequestProducer get = HttpAsyncMethods.createGet(this.clientUri);
            ASYNC_CLIENT.execute(get, new RequestStreamConsumer(this.clientUri), null);
        } catch (Exception e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }
    }

    /**
     * A character consumer to read incoming characters on the request stream.
     */
    static class RequestStreamConsumer extends AsyncCharConsumer<Boolean> {
        private String clientUri;

        /**
         * Constructor.
         *
         * @param clientUri - URI of the FPC Client
         */
        public RequestStreamConsumer(String clientUri) {
            this.clientUri = clientUri;
        }

        @Override
        protected void onResponseReceived(final HttpResponse response) {
        }

        @Override
        protected void onCharReceived(final CharBuffer buf, final IOControl ioctrl) {
            char[] charArray = new char[buf.remaining()];
            System.arraycopy(buf.array(), 0, charArray, 0, buf.remaining());
            CharBuffer charBuffer = CharBuffer.wrap(charArray);
            ParseStream.getInstance().send(new AbstractMap.SimpleEntry<>(clientUri, charBuffer));
        }

        @Override
        protected void releaseResources() {
        }

        @Override
        protected Boolean buildResult(final HttpContext context) {
            return Boolean.TRUE;
        }

    }
}


